@@ -9,7 +9,7 @@ class MarpaX::Languages::M4::Impl::Default::Eval {
     use Types::Common::Numeric -all;
     use MarpaX::Languages::M4::Impl::Default::BaseConversion;
 
-    our $VERSION = '0.012'; # VERSION
+    our $VERSION = '0.013'; # VERSION
 
     our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
@@ -32,7 +32,7 @@ class MarpaX::Languages::M4::Impl::Default::Eval {
     };
 
     method _eval (ConsumerOf['Bit::Vector'] $expression) {
-        return $expression->to_Dec();
+        return $expression;
     }
 
     method _invalidOp (Str $op) {
@@ -249,53 +249,76 @@ class MarpaX::Languages::M4::Impl::Default::Eval {
         }
         return $rc;
     }
-    #
-    # Raw inputs are not allowed to fail. Eventual croaks are left alive.
-    #
+#
+# Raw inputs are not allowed to fail. That's why we always subcall the _radix method
+# whose implementation will use Bit::Vector::Multiply -> this will detect any
+# overflow
+#
     method _decimal (Str $lexeme) {
-        return Bit::Vector->new_Dec( $self->bits, $lexeme + 0 );
+                      #
+                      # decimalNumber ~      _DECDIGITS
+                      #
+        return $self->_radix("0r10:$lexeme");
     }
 
     method _octal (Str $lexeme) {
-        return Bit::Vector->new_Dec( $self->bits, oct($lexeme) );
+                      #
+                      # octalNumber   ~ '0'  _OCTDIGITS
+                      #
+        substr( $lexeme, 0, 1, '' );
+        return $self->_radix("0r8:$lexeme");
     }
 
-    # oct() supportx 0x notation
     method _hex (Str $lexeme) {
-        return Bit::Vector->new_Dec( $self->bits, oct($lexeme) );
+                      #
+                      # hexaNumber    ~ '0x' _HEXDIGITS
+                      #
+        substr( $lexeme, 0, 2, '' );
+        return $self->_radix("0r16:$lexeme");
     }
 
-    # oct() supportx 0b notation
     method _binary (Str $lexeme) {
-        return Bit::Vector->new_Dec( $self->bits, oct($lexeme) );
+                      #
+                      # binaryNumber  ~ '0b' _BINDIGITS
+                      #
+        substr( $lexeme, 0, 2, '' );
+        return $self->_radix( $lexeme, true );
     }
 
-    method _radix (Str $lexeme) {
+    method _radix (Str $lexeme, Bool $binary?) {
                       #
-                      # Per def this is this regexp
+                      # Per def it is this regexp
+                      # C.f. grammar
                       #
-        $lexeme =~ /0r([\d]+):([\da-zA-Z]+)/;
-        my $radix = substr( $lexeme, $-[1], $+[1] - $-[1] );
-        my $digits = lc( substr( $lexeme, $-[2], $+[2] - $-[2] ) )
-            ;         # Because max base is 36
-        if ( $radix == 1 ) {
-
-            # For radix 1, leading zeros are ignored,
-            # and all remaining digits must be 1
-            $digits =~ s/^0*//;
-            if ( $digits =~ /[^1]/ ) {
-                Marpa::R2::Context::bail(
-                    $self->SELF->impl_quote($lexeme) . ': '
-                        . 'for radix 1, digits must be eventual zeroes followed by 1\'s'
-                );
-            }
+        my $radix;
+        my $input = $lexeme;
+        if ( !$binary ) {
+            $lexeme =~ /0r([\d]+):([\da-zA-Z]+)/;
+            $radix = substr( $lexeme, $-[1], $+[1] - $-[1] );
+            $input = substr( $lexeme, $-[2], $+[2] - $-[2] );
         }
-        return Bit::Vector->new_Dec(
-            $self->bits,
-            MarpaX::Languages::M4::Impl::Default::BaseConversion->fr_base(
-                $radix, $digits
-            )
-        );
+        my $error       = false;
+        my $errorString = '';
+        my $rc;
+        try {
+            $rc = MarpaX::Languages::M4::Impl::Default::BaseConversion
+                ->bitvector_fr_base( $self->bits, $radix, $input, $binary );
+        }
+        catch {
+            $error       = true;
+            $errorString = "$_";
+            return;
+        };
+        if ($error) {
+            Marpa::R2::Context::bail( 'Cannot create number '
+                    . $self->SELF->impl_quote($input)
+                    . ' writen in base '
+                    . $self->SELF->impl_quote($radix)
+                    . ' using a bit vector of size '
+                    . $self->SELF->impl_quote( $self->bits ) . ' : '
+                    . $errorString );
+        }
+        return $rc;
     }
 
 }
@@ -314,7 +337,7 @@ MarpaX::Languages::M4::Impl::Default::Eval - Eval Marpa actions
 
 =head1 VERSION
 
-version 0.012
+version 0.013
 
 =head1 AUTHOR
 
