@@ -6,7 +6,7 @@ use Moops;
 
 role MarpaX::Languages::M4::Role::Impl {
 
-    our $VERSION = '0.012'; # VERSION
+    our $VERSION = '0.013'; # VERSION
 
     our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
@@ -34,6 +34,8 @@ role MarpaX::Languages::M4::Role::Impl {
     requires 'impl_debugfile';
     requires 'impl_reloadState';
     requires 'impl_freezeState';
+    requires 'impl_nbInputProcessed';
+    requires 'impl_readFromStdin';
 
     with 'MarpaX::Languages::M4::Role::Builtin';
     with 'MarpaX::Languages::M4::Role::Logger';
@@ -54,7 +56,7 @@ MarpaX::Languages::M4::Role::Impl - M4 implementation role
 
 =head1 VERSION
 
-version 0.012
+version 0.013
 
 =head1 AUTHOR
 
